@@ -6,14 +6,14 @@
  *
  * PLEASE, send any comment, bug report, etc. to the trackers at sourceforge.net
  *
- * Copyright (C) 2006-2007 Benjamin Moll (qjuh@users.sourceforge.net)
+ * Copyright (C) 2006-2011 Benjamin Moll (qjuh@users.sourceforge.net)
  * some portions : Copyright (C) 2004-2006 Nicolas Schoonbroodt (nicolas@ffsa.be)
  *                 Copyright (C) 2004-2006 GRIm@ (thegrima@altern.org).
  * 		   Copyright (C) 2004-2006 Eric Betts (bettse@onid.orst.edu).
  *                 Copyright (C) 2008-2009 Martin Keßler (martin@moegger.de).
  * Windows port  : Copyright (C) 2005-2006 Nicolai Stange (nic-stange@t-online.de)
  * Other portions heavily inspired and copied from gaim sources
- * Copyright (C) 1998-2007 Pidgin developers pidgin.im
+ * Copyright (C) 1998-2011 Pidgin developers pidgin.im
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program (see COPYING); if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA, 02110-1301, USA
  *
  */
 #include "LaTeX.h"
@@ -36,6 +36,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <errno.h>
+#include <regex.h>
 #include <sys/types.h>
 
 #ifndef _WIN32
@@ -321,14 +322,19 @@ static char* get_convert_cmd()
 static gboolean is_blacklisted(char *message)
 {
   char *not_secure[NB_BLACKLIST] = BLACKLIST;
+  int reti;
   int i;
   for (i = 0 ; i < NB_BLACKLIST ; i++)
-  {
-    char *begin_not_secure = malloc((strlen(not_secure[i])+9)*sizeof(char));
-    strcpy(begin_not_secure,"\\begin{");
+  {  
+    regex_t regex;
+    char *begin_not_secure = malloc((strlen(not_secure[i])+18)*sizeof(char));
+    strcpy(begin_not_secure,"\\\\begin\\W*{\\W*");
     strcat(begin_not_secure,not_secure[i]+0x01);
-    strcat(begin_not_secure,"}");
-    if (strstr(message, not_secure[i]) != NULL || strstr(message, begin_not_secure)) return TRUE;
+    strcat(begin_not_secure,"\\W*}");
+    reti = regcomp(&regex, begin_not_secure, 0);
+    reti = regexec(&regex, message, 0, NULL, 0);
+    regfree(&regex);
+    if (strstr(message, not_secure[i]) != NULL || reti!=REG_NOMATCH) return TRUE;  
   }
   return FALSE;
 }
@@ -342,6 +348,7 @@ static gboolean latex_to_image(char *latex, char **file_png)
   char *cmddvipng = NULL;
   char *file_tex;
   char *file_dvi;
+  char *fgcolor, *bgcolor;
   gboolean exec_ok;
   
 //  purple_debug_misc("LaTeX", "starting to transcribe part of the message\n");
@@ -363,6 +370,7 @@ static gboolean latex_to_image(char *latex, char **file_png)
     purple_notify_error(me, "LaTeX", "Error while running LaTeX!", "Couldn't create temporary files.");
     return FALSE;
   }
+  
   strcpy(file_tex, file_tmp);
   strcat(file_tex, ".tex");
   strcpy(file_dvi, file_tmp);
@@ -372,19 +380,57 @@ static gboolean latex_to_image(char *latex, char **file_png)
   unlink(file_tmp);
   free(file_tmp);
   
+  fgcolor = (char*)malloc(16*sizeof(char));
+  if( !strcmp(purple_prefs_get_string("/pidgin/conversations/fgcolor"),""))
+  {
+    strcpy(fgcolor, "0,0,0");
+  }
+  else
+  {
+    char const *pidgin_fgcolor;
+    int rgb;
+    pidgin_fgcolor = purple_prefs_get_string("/pidgin/conversations/fgcolor");
+    purple_debug_info("LaTeX","found foregroundcolor '%s'\n",pidgin_fgcolor);
+    rgb = strtol(pidgin_fgcolor+1,NULL,16);
+    purple_debug_info("LaTeX", "numerical: %d\n", rgb);
+    sprintf(fgcolor,"%d,%d,%d", rgb >> 16, (rgb >> 8) & 0xff, rgb & 0xff);
+  }
+  purple_debug_info("LaTeX", "using '%s' for foreground\n", fgcolor);
+  
+  bgcolor = (char*)malloc(16*sizeof(char));
+  if( !strcmp(purple_prefs_get_string("/pidgin/conversations/bgcolor"),""))
+  {
+    strcpy(bgcolor, "255,255,255");
+  }
+  else
+  {
+    char const *pidgin_bgcolor;
+    int rgb;
+    pidgin_bgcolor = purple_prefs_get_string("/pidgin/conversations/bgcolor");
+    purple_debug_info("LaTeX","found backgroundcolor '%s'\n",pidgin_bgcolor);
+    rgb = strtol(pidgin_bgcolor+1,NULL,16);
+    purple_debug_info("LaTeX", "numerical: %d\n", rgb);
+    sprintf(bgcolor,"%d,%d,%d", rgb >> 16, (rgb >> 8) & 0xff, rgb & 0xff);
+  }
+  purple_debug_info("LaTeX", "using '%s' for background\n", bgcolor);
+  
   if (! (texfile = fopen(file_tex, "w"))) return FALSE;
-  fprintf(texfile, HEADER HEADER_MATH "%s" FOOTER_MATH FOOTER, latex);
+  fprintf(texfile, HEADER "%s" HEADER_COLOR "%s" HEADER_MATH "%s" FOOTER_MATH FOOTER, fgcolor, bgcolor, latex);
   fclose (texfile);
+  free(fgcolor);
   
   tmpdir=getdirname(file_tex);
-  /* generate commands, also new */
+  /* generate commands, also new */  
   char *latexopts[2]={"--interaction=nonstopmode", file_tex};
-  char *dvipngopts[10]={"-bg", "transparent", "-Q", "10", "-T", "tight", "--follow", "-o", *file_png, file_dvi};
+  char *dvipngopts[8]={"-Q", "10", "-T", "tight", "--follow", "-o", *file_png, file_dvi};
   cmdlatex=get_latex_cmd();
   cmddvipng=get_dvipng_cmd();
   
   if (!tmpdir || chdir(tmpdir) || !cmdlatex || !cmddvipng )
   {
+    if( tmpdir ) free(tmpdir);
+    unlink(file_tex);
+    free(file_tex);
     free(file_dvi);
     free(*file_png);
     file_dvi=*file_png=NULL;
@@ -398,7 +444,7 @@ static gboolean latex_to_image(char *latex, char **file_png)
   
 //  purple_debug_misc("LaTeX", "beginning to start latex and dvipng\n");
   
-  exec_ok = !(execute(cmdlatex, latexopts, 2) || execute(cmddvipng, dvipngopts, 10));
+  exec_ok = !(execute(cmdlatex, latexopts, 2) || execute(cmddvipng, dvipngopts, 8));
   
   free(cmdlatex);
   free(cmddvipng);
@@ -416,7 +462,7 @@ static gboolean latex_to_image(char *latex, char **file_png)
   free(file_tex);
   free(file_dvi);
   
-  purple_debug_info("LaTeX", "image creation exited with status '%d'\n", exec_ok);
+  purple_debug_info("LaTeX", "image creation exited with status '%d'\n", !exec_ok);
   
   return exec_ok;
 }
@@ -433,7 +479,7 @@ static gboolean analyse(char **tmp2, char *startdelim, char *enddelim, gboolean
   ptr1 = strstr(*tmp2, startdelim);
   while(ptr1 != NULL)
   {
-    char *tex, *message, *filter, *idstring, *shortcut;
+    char *tex, *tex2, *message, *filter, *idstring, *shortcut;
     gchar *filedata;
     size_t size;
     GError *error = NULL;
@@ -457,7 +503,8 @@ static gboolean analyse(char **tmp2, char *startdelim, char *enddelim, gboolean
     strncpy(tex, &ptr1[strlen(startdelim)], pos2 - pos1 - strlen(startdelim)-strlen(enddelim));
     tex[pos2-pos1-strlen(startdelim)-strlen(enddelim)] = '\0';
     
-    // Pidgin transforms & to &amp; and I make the inverse transformation
+    // Replaced. Using official libpurple-function instead
+    /*/ Pidgin transforms & to &amp; and I make the inverse transformation
     while ( (filter = strstr(tex, FILTER_AND) ) != NULL)
       strcpy(&tex[strlen(tex) - strlen(filter) + 1], &filter[5]);
     
@@ -477,17 +524,23 @@ static gboolean analyse(char **tmp2, char *startdelim, char *enddelim, gboolean
     
     // <br> filter
     while ( (filter = strstr(tex, FILTER_BR) ) != NULL)
-      strcpy(&tex[strlen(tex) - strlen(filter)], &filter[4]);
+      strcpy(&tex[strlen(tex) - strlen(filter)], &filter[4]);//*/
+    
+    tex2 = (char*)malloc((strlen(tex)+1)*sizeof(char));
+    strcpy(tex2, tex);
+    tex = purple_unescape_html(tex2);
     
     purple_debug_misc("LaTeX", "Found LaTeX-Code: %s\n", tex);
     
-    shortcut = (char*)malloc((strlen(tex)+5)*sizeof(char));
+    shortcut = (char*)malloc((strlen(tex2)+5)*sizeof(char));
     strcpy(shortcut,"$$");
-    strcat(shortcut, tex);
+    strcat(shortcut, tex2);
     strcat(shortcut, "$$");
+    free(tex2);
     
     if (smileys && purple_smileys_find_by_shortcut(shortcut)!=NULL) {
       free(tex);
+      free(shortcut);
       ptr1 = strstr(&(*tmp2)[pos2], startdelim);
       continue;
     }
@@ -496,6 +549,7 @@ static gboolean analyse(char **tmp2, char *startdelim, char *enddelim, gboolean
     if (!latex_to_image(tex, &file_png))
     {
       free(tex);
+      free(shortcut);
       return FALSE;
     }
     
@@ -506,6 +560,7 @@ static gboolean analyse(char **tmp2, char *startdelim, char *enddelim, gboolean
     {
       purple_notify_error(me, "LaTeX", "Error while reading the generated image!", error->message);
       g_error_free(error);
+      free(shortcut);
       return FALSE;
     }
     
@@ -518,6 +573,7 @@ static gboolean analyse(char **tmp2, char *startdelim, char *enddelim, gboolean
     if (idimg == 0)
     {
       purple_notify_error(me, "LaTeX", "Error while reading the generated image!", "Failed to store image.");
+      free(shortcut);
       return FALSE;
     }
     
@@ -525,6 +581,7 @@ static gboolean analyse(char **tmp2, char *startdelim, char *enddelim, gboolean
     {
       the_smiley = purple_smiley_new(purple_imgstore_find_by_id(idimg),shortcut);
       pidgin_smiley_add_to_list(the_smiley);
+      free(shortcut);
       
       // making new message
       if ((message = malloc (strlen(*tmp2) + 1)) == NULL)
@@ -538,6 +595,7 @@ static gboolean analyse(char **tmp2, char *startdelim, char *enddelim, gboolean
     }
     else
     {
+      free(shortcut);
       idstring = malloc(10);
       sprintf(idstring, "%d\0", idimg);
     
@@ -545,6 +603,7 @@ static gboolean analyse(char **tmp2, char *startdelim, char *enddelim, gboolean
       if ((message = malloc (strlen(*tmp2) - pos2 + pos1 + strlen(idstring) + strlen(IMG_BEGIN) + strlen(IMG_END) + 1)) == NULL)
       {
         purple_notify_error(me, "LaTeX", "Error while composing the message!", "couldn't make the message.");
+        free(idstring);
         return FALSE;
       }
     
